//
//  CaptureViewController.m
//  Say Cheese Camera
//
//  Created by hirofumi kaneko on 2013/08/12.
//  Copyright (c) 2013年 hirofumi kaneko. All rights reserved.
//

#import "CaptureViewController.h"

@interface CaptureViewController ()
{
    UIView      * captureView_;
    UIImageView * imageView_;
    UILabel     * label_;
    UIButton    * resetBtn_;

    VideoProcessor * videoProcessor_;

    NSString * lmPath_;
    NSString * dicPath_;
    PocketsphinxController * pocketsphinxController_;
    OpenEarsEventsObserver * openEarsEventsObserver_;
}

@end

@implementation CaptureViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

#pragma mark - Life cycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    // キャプチャー表示用
    captureView_ = [[UIView alloc] initWithFrame:[UIScreen mainScreen].bounds];
    [self.view addSubview:captureView_];
    
    // 撮影画像表示用imageView
    imageView_ = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 320, 568)];
    [captureView_ addSubview:imageView_];
    
    // 確認用ラベル
    label_ = [[UILabel alloc] initWithFrame:CGRectMake(0, 528, 320, 40)];
    label_.textAlignment = NSTextAlignmentCenter;
    label_.font = [UIFont italicSystemFontOfSize:14];
    [self.view addSubview:label_];

    __weak CaptureViewController * weakSelf = self;
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    dispatch_async(queue, ^{

        videoProcessor_ = [VideoProcessor new];
        videoProcessor_.delegate = weakSelf;

        dispatch_async(dispatch_get_main_queue(), ^{
            if ([videoProcessor_ setupWithView:captureView_])
            {
                [videoProcessor_ startRunning];
                [weakSelf setUpOpenEars];
            }
        });
    });
}


#pragma mark - VideoProcessor

- (void)videoProcessorTakeCapture:(UIImage *)image
{
    // 画像の向き調整用
    UIGraphicsBeginImageContextWithOptions(image.size, NO, 0);
    [image drawInRect:CGRectMake(0, 0, image.size.width, image.size.height)];
    image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();

    [videoProcessor_ removePreviewLayer];
    [videoProcessor_ stopRunning];
    [self stopListening];
    imageView_.image = image;

    [self saveImage:image];
}

- (void)showResetBtn
{
    resetBtn_ = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    resetBtn_.frame = CGRectMake(0, 0, 80, 44);
    resetBtn_.center = self.view.center;
    [resetBtn_ setTitle:@"Reset" forState:UIControlStateNormal];
    [self.view addSubview:resetBtn_];

    [resetBtn_ addTarget:self action:@selector(reset) forControlEvents:UIControlEventTouchUpInside];
}

- (void)reset
{
    [resetBtn_ removeFromSuperview];
    resetBtn_ = nil;
    imageView_.image = nil;
    [videoProcessor_ addPreviewLayer];
    [videoProcessor_ resume];
    [self setLabelWithText:@"Listen..."];
    [self startListening];
}

#pragma mark - OpenEars

- (void)setUpOpenEars
{
    pocketsphinxController_ = [PocketsphinxController new];
    openEarsEventsObserver_ = [OpenEarsEventsObserver new];
    openEarsEventsObserver_.delegate = self;

    NSArray * words = @[@"Say Cheese", @"Hi Cheese"];
    LanguageModelGenerator * lmGenerator = [LanguageModelGenerator new];
    NSError * error = [lmGenerator generateLanguageModelFromArray:words withFilesNamed:@"OpenEarsDynamicGrammar"];
    if (error.code == noErr)
    {
        NSDictionary * languageGeneratorResults = [error userInfo];
        lmPath_ = [languageGeneratorResults objectForKey:@"LMPath"];
        dicPath_ = [languageGeneratorResults objectForKey:@"DictionaryPath"];
    }
    else
    {
        NSLog(@"Error: %@",[error localizedDescription]);
    }
    [self startListening];
}

- (void)startListening
{
    [pocketsphinxController_ startListeningWithLanguageModelAtPath:lmPath_ dictionaryAtPath:dicPath_ languageModelIsJSGF:NO];
    [self setLabelWithText:@"Listen..."];
}

- (void)stopListening
{
    [pocketsphinxController_ stopListening];
}

- (void)pocketsphinxDidReceiveHypothesis:(NSString *)hypothesis
                        recognitionScore:(NSString *)recognitionScore
                             utteranceID:(NSString *)utteranceID
{
    [self setLabelWithText:hypothesis];
    label_.text = hypothesis;
    if ([hypothesis isEqualToString:@"Hi Cheese"] || [hypothesis isEqualToString:@"Say Cheese"])
    {
        // 認識成功時
        [self setLabelWithText:@"Take a photo"];
        [videoProcessor_ take]; // 撮影
    }
    else
    {
        // 認識失敗時
        [self performSelector:@selector(setLabelWithText:) withObject:@"Listen..." afterDelay:1.5];
    }
}

- (void)setLabelWithText:(NSString *)text
{
    label_.text = text;
}


#pragma mark - Save

- (void)saveImage:(UIImage *)image
{
    ALAssetsLibrary * library = [[ALAssetsLibrary alloc] init];
    [library writeImageToSavedPhotosAlbum:image.CGImage orientation:ALAssetOrientationUp completionBlock:^(NSURL *assetURL, NSError *error)
    {
        [self showResetBtn];
    }];
}


#pragma mark - Gesture

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    CGPoint touchPoint = [[touches anyObject] locationInView:captureView_];
    if (videoProcessor_)
    {
        [videoProcessor_ setFocusWithSize:captureView_.frame.size focusPoint:touchPoint];
    }
}

@end
